/******************************************************************************

	COPYRIGHT(C) JONAS 'SORTIE' TERMANSEN 2011.

	This file is part of Sortix.

	Sortix is free software: you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation, either version 3 of the License, or (at your option) any later
	version.

	Sortix is distributed in the hope that it will be useful, but WITHOUT ANY
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
	FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
	details.

	You should have received a copy of the GNU General Public License along
	with Sortix. If not, see <http://www.gnu.org/licenses/>.

	kernel.cpp
	A common interface shared by all devices that can be printed text to.

******************************************************************************/

#include "platform.h"
#include <libmaxsi/memory.h>
#include "globals.h"
#include "iprintable.h"
#include "log.h"
#include "panic.h"
#include "descriptor_tables.h"
#include "iirqhandler.h"
#include "time.h"
#include "keyboard.h"
#include "multiboot.h"
#include "memorymanagement.h"
#include "scheduler.h"
#include "syscall.h"
#include "pong.h"

void* RunApplication(void* Parameter);

namespace Sortix
{
	void KernelConstructors()
	{
		GLog = new (GLog) Log;
		GPanic = new (GPanic) PanicClass;
		//GShell = new (GShell) Shell;
		//GAstroids = new (GAstroids) Astroids;
	}

	extern "C" void KernelInit(unsigned long Magic, multiboot_info_t* BootInfo)
	{
		// Initialize the global object's by calling their constructors.
		KernelConstructors();

		// Initialize the kernel log.
		GLog->Init();

		// Initialise all the ISRs and the lack of segmentation.
		init_descriptor_tables();

		if ( BootInfo == NULL ) { Panic("kernel.cpp: The bootinfo structure was NULL. Are your bootloader multiboot compliant?"); }

		// Initialize the paging.
		Page::Init(BootInfo);

		// Initialize virtual memory.
		//GLog->Print("Pre-Init'd virtual memory!\n");
		//VirtualMemory::timeInit();
		//GLog->Print("Init'd virtual memory!\n");

		Pong::Init();

		// Initialize the keyboard.
		Keyboard::Init();

		// Initialize system calls.
		//Syscall::Init();

#if 1
		
		Time::Init();

		while ( true ) { }

#endif

		

#if 0

		// Initialize the scheduler.
		Scheduler::Init();

		if ( Scheduler::CreateThread(NULL, RunApplication) == NULL )
		{
			Panic("Could not create a sample thread!");
		}

		// Lastly set up the timer driver and we are ready to run the OS.
		Time::Init();

		// Run the OS.
		Scheduler::MainLoop();
#endif
	}
}
